﻿#region Copyright © 2013 Miracle Business Solutions
// All rights are reserved. Reproduction or transmission in whole or in part,
// any form or by any means, electronic, mechanical or otherwise, is prohibited
// without the prior written consent of the copyright owner.
// 
// Document:	AutoFocus\AutoFocus.cs
// Creation Date:	24/11/2013
// Last Edit:		24/11/2013
// Author:		Moreno Sint Hill
#endregion

using System;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Media;
using AutoFocus;
using Styx.Common;
using Styx.Helpers;
using Styx.Plugins;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Action = Styx.TreeSharp.Action;

namespace Styx
{
    public class AutoFocus : HBPlugin
    {
        // root of our plugins behavior tree
        private Composite Root;

        #region Overrides of HBPlugin
        // standard plugin overrides
        public override string Name
        {
            get { return "AutoFocus - A "; }
        }

        public override string Author
        {
            get { return "Mirabis"; }
        }

        public override Version Version
        {
            get { return new Version(1, 0, 0, 0); }
        }

        /// <summary>
        ///     Initialize() equals Enabled
        ///     ---
        ///     called when user clicks the enable checkbox for the plugin, -or when
        ///     HonorBuddy loads plugins initially if already initialized
        /// </summary>
        public override void Initialize()
        {
            base.Initialize();

            Logging.Write(Colors.Pink, "[AutoFocus] Enabled");

            // create behavior tree and Start (prepares to run when pulsed)
            Root = CreateBehaviorLogic();
            Root.Start(null);
         
        }

        private Form _newtempui;
        /// <summary>
        ///     Called when the button for this CC is pressed.
        /// </summary>
        public override void OnButtonPress()
        {
            //Settings.Instance.Load();
            if (_newtempui == null || _newtempui.IsDisposed || _newtempui.Disposing) _newtempui = new GUI();
            if (_newtempui != null || _newtempui.IsDisposed) _newtempui.ShowDialog();
        }

        public override bool WantButton
        {
            get { return true; }
        }
        /// <summary>
        ///     Dispose() equals Disabled
        ///     ---
        ///     called when user unchecks your plugin -or- when HonorBuddy is closing.  Any
        ///     resource, hook, or event you implemented in Initialize() must be released here
        ///     to avoid the side-effects of your plugin continuing to run after the user disables
        /// </summary>
        public override void Dispose()
        {
            Logging.Write(Colors.Pink, "[AutoFocus] Disabled"); 
            Lua.DoString("RunMacroText('/clearfocus')");
            // stop the behavior tree and release
            Root.Stop(null);
            Root = null;

            base.Dispose();
        }
        /// <summary>
        ///     Pulse() equals Behavior Tree Tick()
        ///     ---
        ///     This will Tick your behavior tree, so does not need to be modified.
        ///     Implement your plugins logic in the CreateBehaviorLogic() method.
        ///     ---
        ///     This style of behavior tree plugin will cause your behavior to tick
        ///     Pulse.  An alternative is to hook one of the predefined TreeHooks
        ///     so your tree runs only during the associated state (Rest, Combat, etc.)
        ///     For an example of tying to a predefined TreeHook see:
        ///     http://www.thebuddyforum.com/honorbuddy-forum/community-developer-forum/114540-answers-treesharp-questions.html#post1114517
        /// </summary>
        public override void Pulse()
        {
            try
            {
                Root.Tick(null);
                // If the last status wasn't running, stop the tree, and restart it.
                if (Root.LastStatus != RunStatus.Running)
                {
                    Root.Stop(null);
                    Root.Start(null);
                }
            }
            catch (Exception e)
            {
                // Restart on any exception.
                Logging.WriteException(e);
                Root.Stop(null);
                Root.Start(null);
                throw;
            }
        }

        #endregion

        #region Plugin Logic
        /// <summary>
        ///     Plugin Logic Goes Here
        ///     --
        ///     This creates the Behavior Tree which represents your plugins logic.
        /// </summary>
        private static Composite CreateBehaviorLogic()
        {// .. logic for behavior tree goes here ..
            return new Decorator(req => StyxWoW.Me.FocusedUnitGuid != AFSettings.Instance.FocusGUID,
                new Action(r =>
                {
                    StyxWoW.Me.SetFocus(AFSettings.Instance.FocusGUID);
                    Logging.Write(Colors.White, "[AutoFocus] Changed focus back to {0} ({1})", AFSettings.Instance.FocusName, AFSettings.Instance.FocusGUID);
                }));
        }


        #endregion

        #region Settings

        internal class AFSettings : Settings
        {
            private static AFSettings _instance;

            public AFSettings()
                : base(
                    Path.Combine(Path.Combine(Utilities.AssemblyDirectory, "Settings"),
                        string.Format("AFSettings{0}.xml", StyxWoW.Me.Name)))
            {
            }

            public static AFSettings Instance
            {
                get { return _instance ?? (_instance = new AFSettings()); }
            }

            [Setting]
            public string FocusName { get; set; }
            [Setting]
            public ulong FocusGUID { get; set; }
        }
        #endregion
    }
}